import csv
import numpy as np
from itertools import izip, takewhile
from scipy.stats import wilcoxon, ttest_rel

reader = csv.reader(open('ml_results.tsv'), delimiter='\t')
classifiers = reader.next()[1:-4]
their_results = {row[0]: {classifier: float(result) if result != '--' else np.nan
                          for classifier, result in zip(classifiers, row[1:-4])}
                 for row in takewhile(lambda row: row[0] != '', reader)}

get_results = lambda classifier: np.array([their_results[benchmark][classifier] 
                                           for benchmark in their_results])

classifiers = ('parRF_caret', 'elm_kernel_matlab', 'rf_caret', 'rforest_R', 
               'svm_C', 'svmRadialCost_caret', 'svmPoly_caret', 
               'svmRadial_caret', 'avNNet_caret', 'C5.0_caret')

results = np.array([get_results(classifier) for classifier in classifiers])

imputation_value = results[0].mean()
print 'Imputation value: {:.0f}%'.format(imputation_value)

for classifier, b in izip(classifiers[1:], results[1:]):
    a = results[0]
    a_nan = np.isnan(a)
    b_nan = np.isnan(b)
    a_imputed = a.copy()
    b_imputed = b.copy()
    a_imputed[a_nan] = imputation_value
    b_imputed[b_nan] = imputation_value
    neither_nan = ~a_nan & ~b_nan
    a = a[neither_nan]
    b = b[neither_nan]
    print '{}\t{:.0g}\t{:.0g}\t{:.0g}'.format(
        classifier, ttest_rel(a_imputed, b_imputed)[1], ttest_rel(a, b)[1], wilcoxon(a, b)[1])